clc;
close all;
clear all;
yalmip('clear')

% Define parameters:
nu = 1;
nx = 2;
h = 0.1;
N = 15;
Tp = N*h;
input.Ts = h;
input.nSteps = 2;

Q = 0.5*eye(nx);
R = eye(nu);
QN = 16.5926*eye(nx);
QN(2,1) = 11.5926;
QN(1,2) = 11.5926;

% Initial state values
x0 = [-0.683; -0.864];

% Initialization of state and control trajectory:
X = zeros(nx, N+1);
U = zeros(nu, N);


figure(1);
subplot(2,2,1);
plot(h*[0:N], X(1,:), '--g');
xlabel('time(s)')
ylabel('x1')
title('State 1')

subplot(2,2,2);
plot(h*[0:N], X(2,:), '--g');
xlabel('time(s)')
ylabel('x2')
title('State 2')

subplot(2,2,3:4);
stairs(h*[0:N-1], U, '--b');
xlabel('time(s)')
ylabel('u')
title('Control')

u = sdpvar(nu*ones(1,N),ones(1,N));
x = sdpvar(nx*ones(1,N+1),ones(1,N+1));

% SET-UP OBJECTIVE:
objective = 0;
for k = 1:N
    objective = objective + h/2*(x{k}'*Q*x{k} + u{k}'*R*u{k});
end
objective = objective + 1/2*x{N+1}'*QN*x{N+1};
    
pause
iter = 0;
DX = 1; DU = 1;
while max(DX,DU) > 1e-6
    
    % SET-UP CONSTRAINTS:
    constraints = [];
    for k = 1:N
        input.x = X(:,k);
        input.u = U(:,k);
        output = RK4_integrator(@ode, input);
        constraints = [constraints, output.value-x{k+1} + output.sensX*(x{k}-X(:,k)) + output.sensU*(u{k}-U(:,k)) == 0];
        constraints = [constraints, -2.0 <= u{k} <= 2.0];
    end
    constraints = [constraints, x{1} == x0];
    
    % SOLVE THE CONVEX PROBLEM:
    opts = sdpsettings('verbose',0,'solver','quadprog');
    solvesdp(constraints, objective, opts);
    
    % GET THE CURRENT VALUES:
    deltaX = X;
    deltaU = U;
    for k = 1:N+1
        deltaX(:,k) = abs(X(:,k) - double(x{k}));
        X(:,k) = double(x{k});
    end
    for k = 1:N
        deltaU(:,k) = abs(U(:,k) - double(u{k}));
        U(:,k) = double(u{k});
    end
    DX = max(max(deltaX));
    DU = max(max(deltaU));
    disp(['correction at iteration ' num2str(iter) ': DX = ' num2str(DX) '; DU = ' num2str(DU)]);
    
    iter = iter+1;
    
    figure(1);
    subplot(2,2,1);
    plot(h*[0:N], X(1,:), '--g');
    xlabel('time(s)')
    ylabel('x1')
    title('State 1')
    
    subplot(2,2,2);
    plot(h*[0:N], X(2,:), '--g');
    xlabel('time(s)')
    ylabel('x2')
    title('State 2')
    
    subplot(2,2,3:4);
    stairs(h*[0:N-1], U, '--b');
    xlabel('time(s)')
    ylabel('u')
    title('Control')

    pause
end

