function [ output ] = RK4_integrator( ode_fun, input )
    x0 = input.x;
    u0 = input.u;
    Ts = input.Ts;
    nSteps = input.nSteps;
    
    nx = length(x0);
    nu = length(u0);
    h = Ts/nSteps;
    
    xEnd = x0;
    A = eye(nx);
    B = zeros(nx,nu);
    for i = 1:nSteps
        x0 = xEnd;
        xEnd = rk4_step(ode_fun,x0,u0,h);
        if ~isfield(input,'sens') || input.sens
            sensX = zeros(nx,nx); sensU = zeros(nx,nu);
            for j = 1:nx
                % finite central differences for states
                xTemp1 = x0; xTemp1(j) = xTemp1(j) + sqrt(eps)/2;
                xTemp1 = rk4_step(ode_fun,xTemp1,u0,h);
                
                xTemp2 = x0; xTemp2(j) = xTemp2(j) - sqrt(eps)/2;
                xTemp2 = rk4_step(ode_fun,xTemp2,u0,h);
                
                sensX(:,j) = (xTemp1 - xTemp2)./sqrt(eps);
            end
            for j = 1:nu
                % finite central differences for controls
                uTemp1 = u0; uTemp1(j) = uTemp1(j) + sqrt(eps)/2;
                xTemp1 = rk4_step(ode_fun,x0,uTemp1,h);
                
                uTemp2 = u0; uTemp2(j) = uTemp2(j) - sqrt(eps)/2;
                xTemp2 = rk4_step(ode_fun,x0,uTemp2,h);
                
                sensU(:,j) = (xTemp1 - xTemp2)./sqrt(eps);
            end
            % propagate sensitivities
            A = sensX*A;
            B = sensX*B + sensU;
        end
    end
    output.value = xEnd;
    if ~isfield(input,'sens') || input.sens
        output.sensX = A;
        output.sensU = B;
    end
end



function x_next = rk4_step(ode_fun,x,u,h)
    k1 = ode_fun(0,x,u);
    k2 = ode_fun(0,x+h/2.*k1,u);
    k3 = ode_fun(0,x+h/2.*k2,u);
    k4 = ode_fun(0,x+h.*k3,u);
    x_next = x + h/6.*(k1+2*k2+2*k3+k4);
end